﻿Imports Syncfusion.Windows.Forms
Imports Syncfusion.Windows.Forms.Edit
Imports Syncfusion.Windows.Forms.Edit.Interfaces
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.IO
Imports System.Linq
Imports System.Reflection
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Forms

Namespace EditControlExample
	Partial Public Class Form1
		Inherits MetroForm

		#Region "Constants"

		' Configuration Path
		Private ConfigPath As String = Path.GetDirectoryName(Application.ExecutablePath) & "\..\..\config.xml"
		' Collection to hold Property Information
		Friend PropertyList As New List(Of String)()
		' Collection to hold Function Information
		Friend FunctionList As New List(Of FunctionInformation)()
		' To hold Function Name
		Private contextPromptLexem As String = ""

		#End Region

		#Region "Constructor"

		Public Sub New()
			InitializeComponent()
			Me.editControl1.Text = "Please choose any dll" & Environment.NewLine & "/* Type 'this' or 'me' and a DOT to see the context choice list. After you type 'this.', select 'Find' (or any of the items)from the context choice list and type a BRACE '(' to see the context prompt */" & ControlChars.Lf & ControlChars.Lf
			Me.editControl1.WordWrap = True
			Me.editControl1.SaveOnClose = False
			Me.editControl1.Configurator.Open(ConfigPath)
			Me.editControl1.ApplyConfiguration("CustomLanguage")
			AddHandler Me.editControl1.ContextChoiceBeforeOpen, AddressOf EditControl1_ContextChoiceBeforeOpen
			AddHandler Me.editControl1.ContextChoiceOpen, AddressOf EditControl1_ContextChoiceOpen
			AddHandler Me.editControl1.ContextChoiceClose, AddressOf EditControl1_ContextChoiceClose
			AddHandler Me.editControl1.ContextPromptOpen, AddressOf EditControl1_ContextPromptOpen
		End Sub

		#End Region

		#Region "Events"

		' Occurs before Function Parameter opens
		Private Sub EditControl1_ContextPromptOpen(ByVal sender As Object, ByVal e As Syncfusion.Windows.Forms.Edit.ContextPromptUpdateEventArgs)
			Me.contextPromptLexem = e.LexemBeforeDropper.Text
			PopulateContextPromptItems(e)
		End Sub

		' Occurs after Context choice open
		Private Sub EditControl1_ContextChoiceOpen(ByVal controller As IContextChoiceController)
			controller.Items.Clear()

			If assembly_Renamed Is Nothing Then
				controller.Items.Add("Chat", "This is Chat")
				controller.Items.Add("Database", "This is Database")
				controller.Items.Add("NewFile", "This is NewFile")
				controller.Items.Add("Find", "This is Find")
				controller.Items.Add("Home", "This is Home")
				controller.Items.Add("PieChart", "This is PieChart")
				controller.Items.Add("Tools", "This is Tools")
			Else
				For Each item As String In Me.PropertyList
					controller.Items.Add(item, "This is " & item)
				Next item
				For Each item As FunctionInformation In Me.FunctionList
					controller.Items.Add(item.FunctionName)
				Next item
			End If
		End Sub

		' Occurs before Context Choice close
		Private Sub EditControl1_ContextChoiceClose(ByVal controller As IContextChoiceController, ByVal dialogresult As DialogResult)
			controller.Items.Clear()
		End Sub

		' Occurs before Content Choice popup is displayed
		Private Sub EditControl1_ContextChoiceBeforeOpen(ByVal sender As Object, ByVal e As CancelEventArgs)
			Dim lex As ILexem
			Dim lexemLine As ILexemLine = Me.editControl1.GetLine(Me.editControl1.CurrentLine)

			'Gets the index of the current word in that line
			Dim ind As Integer = GetContextChoiceCharIndex(lexemLine)

			If ind <= 0 Then
				e.Cancel = True
				Return
			End If

			lex = TryCast(lexemLine.LineLexems(ind - 1), ILexem)

			' If the count is less than '2', do not show the ContextChoice popup
			If lexemLine.LineLexems.Count < 2 Then
				e.Cancel = True
			Else
				' Display ContextChoice popup if the lexem used to invoke them is "this" or "me" only
				If (lex.Text = "this") OrElse (lex.Text = "me") Then
					e.Cancel = False
				Else
					e.Cancel = True
				End If
			End If
		End Sub

'INSTANT VB NOTE: The field assembly was renamed since Visual Basic does not allow fields to have the same name as methods:
		Private assembly_Renamed As System.Reflection.Assembly
		''' <summary>
		''' Gets the Assembly to be loaded
		''' </summary>
		Public WriteOnly Property Assembly() As System.Reflection.Assembly
			Set(ByVal value As System.Reflection.Assembly)
				assembly_Renamed = value
			End Set
		End Property


		''' <summary>
		''' Invoked when click to Choose the assemblies button
		''' </summary>
		Private Sub buttonAdv2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonAdv2.Click
			Dim openFileDialog As New OpenFileDialog()

			openFileDialog.Filter = ".net assemblies (*.exe, *.dll)|*.dll;*.exe|All files (*.*)|*.*"
			openFileDialog.FilterIndex = 1
			openFileDialog.RestoreDirectory = True
			Dim selectedClass As String = Nothing

			PropertyList.Clear()


			If openFileDialog.ShowDialog() = DialogResult.OK Then
				Try
					assembly_Renamed = System.Reflection.Assembly.LoadFrom(openFileDialog.FileName)
				Catch e1 As Exception
					MessageBox.Show("This is not a valid assembly")
					Return
				End Try
				'For Selecting Class file
				Dim typeSelectionDialog As New ClassTypeSelectionDialog()
				'Add all Class file for selected assembly 
				typeSelectionDialog.Assembly = assembly_Renamed

				If typeSelectionDialog.ShowDialog() = DialogResult.OK Then
					Dim tokens() As String = typeSelectionDialog.SelectedType.Split("."c)
					selectedClass = tokens.Last()
				End If
			End If

			If assembly_Renamed IsNot Nothing Then
				For Each [module] As System.Reflection.Module In assembly_Renamed.GetModules()
					For Each type As Type In [module].GetTypes()
						If type.IsSubclassOf(GetType(Control)) AndAlso type.Name = selectedClass Then
							For Each item As PropertyInfo In type.GetProperties()
								If item.PropertyType.IsPublic Then

									If PropertyList.Count < 15 Then
										PropertyList.Add(item.Name)
									Else
										Exit For
									End If
								End If
							Next item
							For Each item As MethodInfo In type.GetMethods()
								If item.IsPublic Then
									If FunctionList.Count < 15 Then
										FunctionList.Add(New FunctionInformation(item.Name, item.GetParameters()))
									Else
										Exit For
									End If
								End If
							Next item
						End If
					Next type
				Next [module]
			Else
				MessageBox.Show("Please select the dll")
			End If
		End Sub

		#End Region

		#Region "Function"

		' Returns the last index of the context choice character - '.' in the current line
		Private Function GetContextChoiceCharIndex(ByVal line As ILexemLine) As Integer
			Dim lastPos As Integer = -1

			For i As Integer = 0 To line.LineLexems.Count - 1
				Dim lex As ILexem = TryCast(line.LineLexems(i), ILexem)

				If lex.Text = "." Then
					lastPos = i
				End If
			Next i

			Return lastPos
		End Function


		' Method that populates the context prompt list based on the lexem invoking it
		Private Sub PopulateContextPromptItems(ByVal e As ContextPromptUpdateEventArgs)
			If assembly_Renamed Is Nothing Then
				Select Case Me.contextPromptLexem
					Case "Chat"
						e.List.Clear()
						e.AddPrompt("Chat(string chatText, int senderID, int receiverID, int connectionID)", "Specify the chat text, the sender information, the receiver information and the connection ID")
						e.AddPrompt("Chat(string chatText, int senderID, bool isBroadCast)", "Specify the chat text, the sender information, and if its a broadcast")
					Case "Database"
						e.List.Clear()
						e.AddPrompt("Database()", "Create a new database")
						e.AddPrompt("Database(string dataBaseName)", "Create a new database with the specified name")
					Case "NewFile"
						e.List.Clear()
						e.AddPrompt("NewFile()", "Create a new file")
						e.AddPrompt("NewFile(string fileName)", "Create a new file with the specified name")
					Case "Find"
						e.List.Clear()
						e.AddPrompt("Find()", "Display the Find dialog")
						e.AddPrompt("Find(string searchText)", "Find occurences of the specified search text")
						e.AddPrompt("Find(string searchText, bool searchUp)", "Find occurences of the specified search text and search up if specified")
					Case "Home"
						e.List.Clear()
						e.AddPrompt("Home()", "Go to the initial application form")
					Case "PieChart"
						e.List.Clear()
						e.AddPrompt("PieChart()", "Create an empty Pie-chart")
						e.AddPrompt("PieChart(string[] sectionNames, int[] sectionAngles)", "Create a Pie-chart containing the specified sections with the specified angles information")
						e.AddPrompt("PieChart(string[] sectionNames, int[] sectionAngles, Color[] sectionColors)", "Create a Pie-chart containing the specified sections with the specified angles and color information")
					Case "Tools"
						e.List.Clear()
						e.AddPrompt("Tools()", "Display the Tools dialog")
						e.AddPrompt("Tools(int ToolID)", "Display the Tools dialog and select the specified Tool")
				End Select
			Else
				For Each item As FunctionInformation In Me.FunctionList
					If Me.contextPromptLexem.ToLower() = item.FunctionName.ToLower() Then
						e.AddPrompt(item.FunctionName, "This is " & item.FunctionName)
						For Each param As ParameterInfo In item.Parameters
							e.AddPrompt(item.FunctionName & "(" & param.Name & ")", param.Name)
						Next param
					End If
				Next item
			End If
		End Sub

		#End Region
	End Class

	''' <summary>
	''' Summary description for TypeSelectionDialog.
	''' </summary>
	Public Class ClassTypeSelectionDialog
		Inherits System.Windows.Forms.Form

		Private WithEvents bOK As System.Windows.Forms.Button
		Private WithEvents bCancel As System.Windows.Forms.Button
		Private label1 As System.Windows.Forms.Label
		Private WithEvents typeList As System.Windows.Forms.ListBox
		''' <summary>
		''' Required designer variable.
		''' </summary>
		Private components As System.ComponentModel.Container = Nothing

		Public Sub New()
			'
			' Required for Windows Form Designer support
			'
			InitializeComponent()

			'
			' TODO: Add any constructor code after InitializeComponent call
			'
		End Sub

		''' <summary>
		''' Clean up any resources being used.
		''' </summary>
		Protected Overrides Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				If components IsNot Nothing Then
					components.Dispose()
				End If
			End If
			MyBase.Dispose(disposing)
		End Sub

		#Region "Windows Form Designer generated code"
		''' <summary>
		''' Required method for Designer support - do not modify
		''' the contents of this method with the code editor.
		''' </summary>
		Private Sub InitializeComponent()
			Me.typeList = New System.Windows.Forms.ListBox()
			Me.bOK = New System.Windows.Forms.Button()
			Me.bCancel = New System.Windows.Forms.Button()
			Me.label1 = New System.Windows.Forms.Label()
			Me.SuspendLayout()
			' 
			' typeList
			' 
			Me.typeList.Location = New System.Drawing.Point(0, 64)
			Me.typeList.Name = "typeList"
			Me.typeList.Size = New System.Drawing.Size(296, 147)
			Me.typeList.TabIndex = 0
'INSTANT VB NOTE: The following InitializeComponent event wireup was converted to a 'Handles' clause:
'ORIGINAL LINE: this.typeList.DoubleClick += new System.EventHandler(this.typeList_DoubleClick);
'INSTANT VB NOTE: The following InitializeComponent event wireup was converted to a 'Handles' clause:
'ORIGINAL LINE: this.typeList.SelectedIndexChanged += new System.EventHandler(this.typeList_SelectedIndexChanged);
			' 
			' bOK
			' 
			Me.bOK.Enabled = False
			Me.bOK.Location = New System.Drawing.Point(128, 232)
			Me.bOK.Name = "bOK"
			Me.bOK.TabIndex = 1
			Me.bOK.Text = "OK"
'INSTANT VB NOTE: The following InitializeComponent event wireup was converted to a 'Handles' clause:
'ORIGINAL LINE: this.bOK.Click += new System.EventHandler(this.bOK_Click);
			' 
			' bCancel
			' 
			Me.bCancel.Location = New System.Drawing.Point(208, 232)
			Me.bCancel.Name = "bCancel"
			Me.bCancel.TabIndex = 2
			Me.bCancel.Text = "Cancel"
'INSTANT VB NOTE: The following InitializeComponent event wireup was converted to a 'Handles' clause:
'ORIGINAL LINE: this.bCancel.Click += new System.EventHandler(this.bCancel_Click);
			' 
			' label1
			' 
			Me.label1.Location = New System.Drawing.Point(8, 16)
			Me.label1.Name = "label1"
			Me.label1.Size = New System.Drawing.Size(280, 32)
			Me.label1.TabIndex = 3
			Me.label1.Text = "Select the Control type to trace"
			' 
			' TypeSelectionDialog
			' 
			Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
			Me.ClientSize = New System.Drawing.Size(292, 266)
			Me.Controls.AddRange(New System.Windows.Forms.Control() { Me.label1, Me.bCancel, Me.bOK, Me.typeList})
			Me.Name = "TypeSelectionDialog"
			Me.Text = "Public Classes"
			Me.ResumeLayout(False)

		End Sub
		#End Region

'INSTANT VB NOTE: The field assembly was renamed since Visual Basic does not allow fields to have the same name as methods:
		Private assembly_Renamed As System.Reflection.Assembly
		Public WriteOnly Property Assembly() As System.Reflection.Assembly
			Set(ByVal value As System.Reflection.Assembly)
				assembly_Renamed = value
				PopulateTypeList()
			End Set
		End Property

		Private Sub PopulateTypeList()
			typeList.Items.Clear()

			For Each [module] As System.Reflection.Module In assembly_Renamed.GetModules()
				For Each type As Type In [module].GetTypes()
					If type.IsSubclassOf(GetType(Control)) Then
						typeList.Items.Add(type.FullName)
					End If
				Next type
			Next [module]
		End Sub

'INSTANT VB NOTE: The field selectedType was renamed since Visual Basic does not allow fields to have the same name as methods:
		Private selectedType_Renamed As String

		Public ReadOnly Property SelectedType() As String
			Get
				Return selectedType_Renamed
			End Get
		End Property

		Private Sub typeList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles typeList.DoubleClick
			selectedType_Renamed = DirectCast(typeList.SelectedItem, String)
			DialogResult = System.Windows.Forms.DialogResult.OK
		End Sub

		Private Sub bCancel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles bCancel.Click
			DialogResult = System.Windows.Forms.DialogResult.Cancel
		End Sub

		Private Sub bOK_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles bOK.Click
			selectedType_Renamed = DirectCast(typeList.SelectedItem, String)
			DialogResult = System.Windows.Forms.DialogResult.OK
		End Sub

		Private Sub typeList_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles typeList.SelectedIndexChanged
			bOK.Enabled = True
		End Sub
	End Class

	#Region "Function Information Class"

	''' <summary>
	''' Class to hold Function Information
	''' </summary>
	Public Class FunctionInformation
		Public Sub New(ByVal Fname As String, ByVal Parameterslist() As ParameterInfo)
			FunctionName = Fname
			Parameters = Parameterslist
		End Sub

		Private m_FunctionName As String
		''' <summary>
		''' Gets/Sets the Function Name
		''' </summary>
		Public Property FunctionName() As String
			Get
				Return m_FunctionName
			End Get
			Set(ByVal value As String)
				m_FunctionName = value
			End Set
		End Property

		Private m_Parameters() As ParameterInfo
		''' <summary>
		''' Gets/Sets the Parameter Information
		''' </summary>
		Public Property Parameters() As ParameterInfo()
			Get
				Return m_Parameters
			End Get
			Set(ByVal value As ParameterInfo())
				m_Parameters = value
			End Set
		End Property
	End Class

	#End Region
End Namespace
